@@ -27,17 +27,21 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "gtkmathview.h"
+#include <gdome.h>
+#include "gtkmathview_gmetadom.h"
 #include "guiGTK.h"
 
+extern GdomeDOMString* find_hyperlink(GdomeElement*, const char*, const char*);
+extern GdomeElement* find_xref_element(GdomeElement*);
+extern GdomeElement* find_common_ancestor(GdomeElement*, GdomeElement*);
+extern GdomeElement* find_self_or_ancestor(GdomeElement*, const char*, const char*);
+
 #define XLINK_NS_URI "http://www.w3.org/1999/xlink"
 
 static GtkWidget* window;
 static GtkWidget* main_area;
 static GtkWidget* scrolled_area;
 static GtkWidget* status_bar;
-static GtkMenuItem* anti_aliasing_item;
-static GtkMenuItem* transparency_item;
 static GdkCursor* normal_cursor;
 static GdkCursor* link_cursor;  
 static gboolean   semantic_selection = FALSE;
@@ -53,13 +57,10 @@ static GtkWidget* get_main_menu(void);
 static void file_open(GtkWidget*, gpointer);
 static void file_re_open(GtkWidget*, gpointer);
 static void file_close(GtkWidget*, gpointer);
-static void options_font_manager(GtkWidget*, FontManagerId);
 static void options_set_font_size(GtkWidget*, gpointer);
 static void options_change_font_size(GtkWidget*, gboolean);
 static void options_verbosity(GtkWidget*, guint);
 static void options_selection(GtkWidget*, gboolean);
-static void options_anti_aliasing(GtkWidget*, gpointer);
-static void options_transparency(GtkWidget*, gpointer);
 static void selection_delete(GtkWidget*, gpointer);
 static void selection_parent(GtkWidget*, gpointer);
 static void selection_reset(GtkWidget*, gpointer);
@@ -84,9 +85,6 @@ static GtkItemFactoryEntry menu_items[] = {
   { "/Options/Default Font Size/sep1", NULL, NULL,                  0,  "<Separator>" },
   { "/Options/Default Font Size/Larger", "<control>2", options_change_font_size, TRUE, NULL },
   { "/Options/Default Font Size/Smaller", "<control>1", options_change_font_size, FALSE, NULL },
-  { "/Options/Font Manager",           NULL, NULL,                  0,  "<Branch>" },
-  { "/Options/Font Manager/_GTK",      NULL, options_font_manager,  FONT_MANAGER_GTK, "<RadioItem>" },
-  { "/Options/Font Manager/_Type 1",   NULL, options_font_manager,  FONT_MANAGER_T1, "/Options/Font Manager/GTK" },
   { "/Options/Verbosity",              NULL, NULL,                  0,  "<Branch>" },
   { "/Options/Verbosity/_Errors",      NULL, options_verbosity,     0,  "<RadioItem>" },
   { "/Options/Verbosity/_Warnings",    NULL, options_verbosity,     1,  "/Options/Verbosity/Errors" },
@@ -95,9 +93,6 @@ static GtkItemFactoryEntry menu_items[] = {
   { "/Options/sep1",                   NULL, NULL,                  0,  "<Separator>" },
   { "/Options/Selection/Structure",    NULL, options_selection,     0,  "<RadioItem>" },
   { "/Options/Selection/Semantics",    NULL, options_selection,     1,  "/Options/Selection/Structure" },
-  { "/Options/sep2",                   NULL, NULL,                  0,  "<Separator>" },
-  { "/Options/_Anti Aliasing",         NULL, options_anti_aliasing, 0,  "<ToggleItem>" },
-  { "/Options/_Transparency",          NULL, options_transparency,  0,  "<ToggleItem>" },
 
   { "/_Help" ,        NULL,         NULL,          0, "<LastBranch>" },
   { "/Help/About...", NULL,         help_about,    0, NULL }
@@ -148,7 +143,7 @@ load_error_msg(const char* name)
 }
 
 void
-GUI_init(int* argc, char*** argv, char* title, guint width, guint height)
+GUI_init(int* argc, char*** argv, char* title, guint width, guint height, gint logLevel)
 {
   gtk_init(argc, argv);
 
@@ -161,6 +156,9 @@ GUI_init(int* argc, char*** argv, char* title, guint width, guint height)
 		  (GtkSignalFunc) gtk_main_quit, NULL);
   
   create_widget_set();
+  g_assert(main_area != NULL);
+  g_assert(GTK_IS_MATH_VIEW(main_area));
+  gtk_math_view_set_log_verbosity(GTK_MATH_VIEW(main_area), logLevel);
 
   gtk_widget_show(window);
 
@@ -171,11 +169,14 @@ GUI_init(int* argc, char*** argv, char* title, guint width, guint height)
 void
 GUI_uninit()
 {
+  /*gtk_widget_unref(GTK_WIDGET(window));*/
 }
 
 int
 GUI_load_document(const char* name)
 {
+  GdomeException exc = 0;
+  GdomeElement* root;
   GtkMathView* math_view;
 
   g_return_val_if_fail(name != NULL, -1);
@@ -184,10 +185,7 @@ GUI_load_document(const char* name)
 
   math_view = GTK_MATH_VIEW(main_area);
 
-  if (!gtk_math_view_load_uri(math_view, name)) {
-    load_error_msg(name);
-    return -1;
-  }
+  gtk_math_view_load_uri(math_view, name);
 
   if (name != doc_name) {
     if (doc_name != NULL) g_free(doc_name);
@@ -226,37 +224,6 @@ GUI_run()
   gtk_main();
 }
 
-void
-GUI_set_font_manager(FontManagerId id)
-{
-  gboolean t1;
-  GtkMathView* math_view;
-
-  g_return_if_fail(id != FONT_MANAGER_UNKNOWN);
-  g_return_if_fail(main_area != NULL);
-  g_return_if_fail(GTK_IS_MATH_VIEW(main_area));
-
-  t1 = id == FONT_MANAGER_T1;
-
-  math_view = GTK_MATH_VIEW(main_area);
-
-  gtk_math_view_freeze(math_view);
-
-  if (id != gtk_math_view_get_font_manager_type(math_view))
-    gtk_math_view_set_font_manager_type(math_view, id);
-
-  gtk_widget_set_sensitive(anti_aliasing_item, t1);
-  gtk_widget_set_sensitive(transparency_item, t1);
-
-  if (t1)
-    {
-      gtk_math_view_set_anti_aliasing(math_view, GTK_CHECK_MENU_ITEM(anti_aliasing_item)->active);
-      gtk_math_view_set_transparency(math_view, GTK_CHECK_MENU_ITEM(transparency_item)->active);
-    }
-
-  gtk_math_view_thaw(math_view);
-}
-
 static void
 store_filename(GtkFileSelection* selector, GtkWidget* user_data)
 {
@@ -310,27 +277,6 @@ file_open(GtkWidget* widget, gpointer data)
   gtk_widget_show (fs);
 }
 
-static void
-options_font_manager(GtkWidget* widget, FontManagerId id)
-{
-  g_return_if_fail(id != FONT_MANAGER_UNKNOWN);
-  GUI_set_font_manager(id);
-}
-
-static void
-options_anti_aliasing(GtkWidget* widget, gpointer data)
-{
-  gboolean aa = gtk_math_view_get_anti_aliasing(GTK_MATH_VIEW(main_area));
-  gtk_math_view_set_anti_aliasing(GTK_MATH_VIEW(main_area), !aa);
-}
-
-static void
-options_transparency(GtkWidget* widget, gpointer data)
-{
-  gboolean t = gtk_math_view_get_transparency(GTK_MATH_VIEW(main_area));
-  gtk_math_view_set_transparency(GTK_MATH_VIEW(main_area), !t);
-}
-
 static void
 options_verbosity(GtkWidget* widget, guint level)
 {
@@ -395,7 +341,7 @@ help_about(GtkWidget* widget, gpointer data)
   GtkWidget* ok;
 
   dialog = gtk_dialog_new();
-  label = gtk_label_new("\n    MathML Viewer    \n    Copyright (C) 2000-2003 Luca Padovani    \n");
+  label = gtk_label_new("\n    MathML Viewer    \n    Copyright (C) 2000-2004 Luca Padovani    \n");
   ok = gtk_button_new_with_label("Close");
 
   /* gtk_signal_connect_object (GTK_OBJECT (ok), "clicked",
@@ -491,14 +437,15 @@ options_set_font_size(GtkWidget* widget, gpointer data)
 
 #if defined(HAVE_GMETADOM)
 static void
-element_over(GtkMathView* math_view, GdomeElement* elem, gint state)
+element_over(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 {
   GdomeDOMString* link = NULL;
 
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(GTK_IS_MATH_VIEW(math_view));
+  g_return_if_fail(event != NULL);
 
-  link = find_hyperlink(elem, XLINK_NS_URI, "href");
+  link = find_hyperlink(event->id, XLINK_NS_URI, "href");
   if (link != NULL)
     gdk_window_set_cursor(GTK_WIDGET(math_view)->window, link_cursor);
   else
@@ -509,12 +456,13 @@ element_over(GtkMathView* math_view, GdomeElement* elem, gint state)
 }
 
 static void
-select_begin(GtkMathView* math_view, GdomeElement* elem, gint state)
+select_begin(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 {
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(GTK_IS_MATH_VIEW(math_view));
+  g_return_if_fail(event != NULL);
 
-  if (elem != NULL)
+  if (event->id != NULL)
     {
       GdomeException exc = 0;
 
@@ -530,7 +478,7 @@ select_begin(GtkMathView* math_view, GdomeElement* elem, gint state)
 
       if (semantic_selection)
 	{
-	  GdomeElement* new_elem = find_xref_element(elem);
+	  GdomeElement* new_elem = find_xref_element(event->id);
 	  if (new_elem != NULL)
             {
 	      gdome_el_ref(new_elem, &exc);
@@ -540,11 +488,11 @@ select_begin(GtkMathView* math_view, GdomeElement* elem, gint state)
 	}
       else
 	{
-	  gdome_el_ref(elem, &exc);
+	  gdome_el_ref(event->id, &exc);
 	  g_assert(exc == 0);
-	  gdome_el_ref(elem, &exc);
+	  gdome_el_ref(event->id, &exc);
 	  g_assert(exc == 0);
-          first_selected = root_selected = elem;
+          first_selected = root_selected = event->id;
 	}
 
       if (root_selected != NULL)
@@ -555,12 +503,13 @@ select_begin(GtkMathView* math_view, GdomeElement* elem, gint state)
 }
 
 static void
-select_over(GtkMathView* math_view, GdomeElement* elem, gint state)
+select_over(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 {
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(GTK_IS_MATH_VIEW(math_view));
+  g_return_if_fail(event != NULL);
 
-  if (first_selected != NULL && elem != NULL)
+  if (first_selected != NULL && event->id != NULL)
     {
       GdomeException exc = 0;
 
@@ -576,7 +525,7 @@ select_over(GtkMathView* math_view, GdomeElement* elem, gint state)
 
       if (semantic_selection)
 	{
-	  GdomeElement* new_root = find_common_ancestor(first_selected, elem);
+	  GdomeElement* new_root = find_common_ancestor(first_selected, event->id);
 	  if (new_root != NULL)
 	    {
 	      root_selected = find_xref_element(new_root);
@@ -587,26 +536,21 @@ select_over(GtkMathView* math_view, GdomeElement* elem, gint state)
 	    root_selected = NULL;
 	}
       else
-        root_selected = find_common_ancestor(first_selected, elem);
+        root_selected = find_common_ancestor(first_selected, event->id);
 
       if (root_selected != NULL)
-	{
-	  gint x, y, width, height, depth;
-	  gtk_math_view_select(math_view, root_selected);
-	  gtk_math_view_get_element_coords(math_view, root_selected, &x, &y);
-	  gtk_math_view_get_element_bounding_box(math_view, root_selected, &width, &height, &depth);
-	  printf("selected element at %d %d bounding box %d %d %d\n", x, y, width, height, depth);
-	}
+	gtk_math_view_select(math_view, root_selected);
 
       gtk_math_view_thaw(math_view);
     }
 }
 
 static void
-select_end(GtkMathView* math_view, GdomeElement* elem, gint state)
+select_end(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 {
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(GTK_IS_MATH_VIEW(math_view));
+  g_return_if_fail(event != NULL);
 
   if (first_selected != NULL)
     {
@@ -644,27 +588,55 @@ select_abort(GtkMathView* math_view)
 }
 
 static void
-click(GtkMathView* math_view, GdomeElement* elem, gint state)
+click(GtkMathView* math_view, const GtkMathViewModelEvent* event)
 {
+  gboolean res;
+  gint index;
+  GdomeElement* elem;
   GdomeException exc;
   GdomeDOMString* name;
   GdomeDOMString* ns_uri;
   GdomeElement* p;
 
-  g_return_if_fail(math_view != NULL);
+  gint w, h;
+  gtk_math_view_get_size(math_view, &w, &h);
+  printf("click signal %d %d\n", w, h);
 
-  if (elem != NULL)
+  g_return_if_fail(math_view != NULL);
+  g_return_if_fail(event != NULL);
+
+  index = -1;
+  if (gtk_math_view_get_char_at(math_view, event->x, event->y, &elem, &index, NULL, NULL))
+    {
+      /*printf("get_char_at: (event->id = %p, elem = %p) %d\n", event->id, elem, index);*/
+      gtk_math_view_set_cursor(math_view, event->id, index);
+      gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_ON);
+    }
+  else
+    {
+      /*printf("get_char_at: failed\n");*/
+      gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_OFF);
+    }
+
+  if (event->id != NULL)
+    {
+      name = gdome_el_nodeName(event->id, &exc);
+      g_assert(exc == 0);
+      printf("node name: %s\n", name->str);
+    }
+
+  if (event->id != NULL)
     {
       GdomeElement* action;
-      GdomeDOMString* href = find_hyperlink(elem, XLINK_NS_URI, "href");
+      GdomeDOMString* href = find_hyperlink(event->id, XLINK_NS_URI, "href");
       if (href != NULL)
 	{
-/* 	  printf("hyperlink %s\n", href->str); */
+ 	  printf("hyperlink %s\n", href->str);
 	  gdome_str_unref(href);
 	}
 
-      action = find_self_or_ancestor(elem, MATHML_NS_URI, "maction");
-/*       printf("action? %p\n", action); */
+      action = find_action_element(event->id);
+      printf("action? %p\n", action);
       if (action != NULL)
 	{
 	  gtk_math_view_freeze(math_view);
@@ -693,6 +665,7 @@ create_widget_set()
 {
   GtkWidget* main_vbox;
   GtkWidget* menu_bar;
+  GtkWidget* frame;
 
   main_vbox = gtk_vbox_new(FALSE, 1);
   gtk_container_border_width(GTK_CONTAINER(main_vbox), 1);
@@ -703,91 +676,54 @@ create_widget_set()
   gtk_box_pack_start(GTK_BOX(main_vbox), menu_bar, FALSE, TRUE, 0);
   gtk_widget_show(menu_bar);
 
-  main_area = gtk_math_view_new(NULL, NULL);
-  gtk_widget_show(main_area);
-
-  /* gtk_signal_connect_object (GTK_OBJECT (main_area),
-			     "select_begin", GTK_SIGNAL_FUNC (select_begin),
-			     (gpointer) main_area); */
-
-  g_signal_connect_swapped(GTK_OBJECT (main_area),
-		  "select_begin", 
-		  G_CALLBACK(select_begin),
-		  (gpointer) main_area);
-
-
-
-  /* gtk_signal_connect_object (GTK_OBJECT (main_area),
-			     "select_over", GTK_SIGNAL_FUNC (select_over),
-			     (gpointer) main_area); */
-
-  g_signal_connect_swapped(GTK_OBJECT (main_area),
-		  "select_over", 
-		  G_CALLBACK(select_over),
-		  (gpointer) main_area);
-
-  /*gtk_signal_connect_object (GTK_OBJECT (main_area),
-			     "select_end", GTK_SIGNAL_FUNC (select_end),
-			     (gpointer) main_area); */
-
- g_signal_connect_swapped(GTK_OBJECT (main_area),
-		 "select_end", 
-		 G_CALLBACK(select_end),
-		 (gpointer) main_area);
-
-
-  /*gtk_signal_connect_object (GTK_OBJECT (main_area),
-			     "select_abort", GTK_SIGNAL_FUNC (select_abort),
-			     (gpointer) main_area);*/
- 
- g_signal_connect_swapped(GTK_OBJECT (main_area),
-		 "select_abort", 
-		 G_CALLBACK(select_abort),
-		 (gpointer) main_area);
-
-
-  /*gtk_signal_connect_object (GTK_OBJECT (main_area),
-			     "element_over", GTK_SIGNAL_FUNC (element_over),
-			     (gpointer) main_area);*/
-
- g_signal_connect_swapped(GTK_OBJECT (main_area),
-		 "element_over", 
-		 G_CALLBACK(element_over),
-		 (gpointer) main_area);
-
-
-  /*gtk_signal_connect_object (GTK_OBJECT (main_area), 
-			     "click", GTK_SIGNAL_FUNC(click),
-			     (gpointer) main_area);*/
-
-  g_signal_connect_swapped(GTK_OBJECT (main_area),
-		  "click", 
-		  G_CALLBACK(click),
-		  (gpointer) main_area);
-
-  gtk_widget_add_events(GTK_WIDGET(main_area),
-			GDK_BUTTON_PRESS_MASK
-			| GDK_BUTTON_RELEASE_MASK
-			| GDK_POINTER_MOTION_MASK);
-
   scrolled_area = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_area),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_area),
-		  			GTK_SHADOW_IN);
+				      GTK_SHADOW_IN);
   gtk_widget_show(scrolled_area);
-  gtk_container_add(GTK_CONTAINER(scrolled_area), main_area);
   gtk_box_pack_start(GTK_BOX(main_vbox), scrolled_area, TRUE, TRUE, 0);
 
+  main_area = gtk_math_view_new(NULL, NULL);
+  gtk_widget_show(main_area);
+  gtk_container_add(GTK_CONTAINER(scrolled_area), main_area);
+
+  g_signal_connect(GTK_OBJECT (main_area),
+		   "select_begin", 
+		   G_CALLBACK(select_begin),
+		   (gpointer) main_area);
+
+  g_signal_connect(GTK_OBJECT (main_area),
+		   "select_over", 
+		   G_CALLBACK(select_over),
+		   (gpointer) main_area);
+
+  g_signal_connect(GTK_OBJECT (main_area),
+		   "select_end", 
+		   G_CALLBACK(select_end),
+		   (gpointer) main_area);
+ 
+  g_signal_connect(GTK_OBJECT (main_area),
+		   "select_abort", 
+		   G_CALLBACK(select_abort),
+		   (gpointer) main_area);
+
+  g_signal_connect(GTK_OBJECT (main_area),
+		   "element_over", 
+		   G_CALLBACK(element_over),
+		   (gpointer) main_area);
+  
+  g_signal_connect(GTK_OBJECT (main_area),
+		   "click", 
+		   G_CALLBACK(click),
+		   (gpointer) main_area);
+
   status_bar = gtk_statusbar_new();
   gtk_widget_show(status_bar);
   gtk_box_pack_start(GTK_BOX(main_vbox), status_bar, FALSE, TRUE, 0);
   statusbar_context = gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar), "filename");
 
   gtk_widget_show(main_vbox);
-
-  if (gtk_math_view_get_anti_aliasing(GTK_MATH_VIEW(main_area)))
-    gtk_menu_item_activate(anti_aliasing_item);
 }
 
 GtkWidget*
@@ -807,11 +743,5 @@ get_main_menu()
 
   gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 
-  menu_item = gtk_item_factory_get_widget(item_factory, "/Options/Anti Aliasing");
-  anti_aliasing_item = GTK_MENU_ITEM(menu_item);
-
-  menu_item = gtk_item_factory_get_widget(item_factory, "/Options/Transparency");
-  transparency_item = GTK_MENU_ITEM(menu_item);
-
   return gtk_item_factory_get_widget(item_factory, "<main>");
 }
